@@ -1,5 +1,10 @@
 ###############################################################################
-# Author: M. Baaske
+# Author:  M. Baaske
+# Date:	   2018/06/15	
+# File:    spheroid.R: 
+# 
+# Comment: simulation, intersections and visualization of spheroid systems
+# 
 ###############################################################################
 
 # angle in the section plane relative to z axis in 3D
@@ -76,12 +81,12 @@ sectionProfiles <- function(size,angle,type=c("prolate","oblate")) {
 #'
 #' Reinitialize spheroid system after R workspace reloading
 #'
-#' The internally stored sphere system has to be reinitialized
-#' after R workspace reloading. Calling this function is needed only in case one desires to get profile
-#' sections of the spheroid system again which has been previously stored as an R (list) object and afterwards
+#' The internal data structure has to be reinitialized if a spheroid system was reloaded to the R workspace.
+#' Calling this function is needed only in case one desires to get profile sections of the spheroid
+#' system again which has been previously stored as an R (list) object and afterwards
 #' reloaded.
 #'
-#' @param S      sphere system
+#' @param S      spheroid system
 #' @param pl	 print level
 #' @param mu	 main orientation vector
 #' @return		 \code{NULL}
@@ -141,17 +146,21 @@ getSpheroidSystem <- function(S) {
 #' which correspond to the lower and upper points in each direction. If the argument \code{box} has
 #' only one element, i.e. \code{list(c(0,1)}, the same extent is used for the other dimensions.
 #' If \code{rjoint} names a joint random generating function then argument \code{size} is ignored.
-#' For the purpose of exact simulation setting \code{size} equal to \code{rbinorm} declares a bivarite
+#' 
+#' For the purpose of exact simulation setting \code{size} equal to \code{rbinorm} declares a bivariate
 #' size-shape distribution which leads to a log normal distributed semi-major axis \code{a} and a scaled
 #' semi-minor axis length \code{c}. If \eqn{[X,Y]} follow a bivariate normal distribution with correlation parameter
 #' \eqn{\rho} then \eqn{a=exp(x)} defines the sample semi-major axis length together with the scaled semi-minor
-#' axis length \eqn{c=a*s} and shape parameter set to \eqn{s=1/(1+exp(-y))}. The parameter \eqn{\rho} defines
-#' the degree of correlation between the semi-axes lengths which must be provided as part of the list of simulation
-#' parameters \code{theta}. The method of exact simulation is tailored to the above described model. For a general
-#' approach please see the given reference below. Other (univariate)  major-axis lengths types include the beta,
-#' gamma, lognormal and uniform distribution where the shape factor which determines the minor-axis length either
-#' follows a beta distribution or is set to a constant. Despite the case of constant size simulations all other
-#' simulations are done as perfect simulations.
+#' axis length \eqn{c=a*s} and shape parameter set to \eqn{s=1/(1+exp(-y))}. Thus the parameter \eqn{\rho} defines the
+#' dependence of the semi-major and semi-minor axis lengths and must be provided as part of the list of simulation
+#' parameters given by \code{theta}. The method of exact simulation is tailored to the above parametric model. For a general
+#' approach please see the given reference below. Other (univariate)  semi-major axis lengths distributions include the `beta`,
+#' `gamma`, `lognormal` and uniform where the shape factor, which sets the semi-minor axis length, either
+#' follows a `beta` distribution or is set to a constant factor. In case of a biavariate normal major-axis size distribution
+#' (\code{size}="\code{rbinorm}"), setting \code{shape="rbeta"} (default option is \code{"const"}) generates a `beta` distributed
+#' second shape factor which is multiplied to the first shorter semi-axis in order to calculate the second one. This
+#' allows the simulation of general ellipsoids. Only simulations done by \code{rbinorm} can use the exact simulation type
+#' if \code{perfect=TRUE} otherwise it is ignored.
 #'
 #' The argument \code{pl} denotes the print level of output information during simulation.
 #' Currently, only \code{pl}=0 for no output and \code{pl}>100 for some additional info is implemented.
@@ -245,7 +254,7 @@ simSpheroidSystem <- function(theta, lam, size="const", shape="const", orientati
 				"rdist"=list("size"=size, "shape"=shape,"orientation"=orientation),
 				"box"=box,"pl"=pl,"mu"=mu,"rho"=.GlobalEnv,"label"=label,"perfect"=as.integer(perfect))
 
-		if(cond$rdist$size %in% c("const","rbinorm")) {
+		if(cond$rdist$size %in% c("const","rbinorm","rbinorm_unequal")) {
 			structure(.Call(C_EllipsoidSystem, theta, cond), box = box)
 		} else if(exists(cond$rdist$size, mode="function")) {
 			fargs <- names(formals(cond$rdist$size))
@@ -345,8 +354,9 @@ coefficientMatrixSpheroids <- function(breaks, stype=c("prolate","oblate"),
 #' @param S		 list of spheroids, see \code{\link{simSpheroidSystem}}
 #' @param d 	 distance of intersecting plane to the origin
 #' @param n 	 normal vector of intersting plane
-#' @param intern 	\code{intern=FALSE} (default) return all section profiles otherwise
-#' 					only those which have their centers inside the intersection window
+#' @param intern logical, \code{FALSE} (default), return all section profiles otherwise
+#' 				 only those which have their centers inside the intersection window (if the 
+#' 				 intersected spheroid system had been simulated using exact simulation)
 #' @return list of size, shape and angle of section profiles
 verticalSection <- function(S,d,n=c(0,1,0),intern=FALSE) {
 	stopifnot(is.logical(intern))
@@ -359,11 +369,26 @@ verticalSection <- function(S,d,n=c(0,1,0),intern=FALSE) {
 	structure(
 	    list("A"=A,
 			 "S"=sapply(ss,"[[",3),
-		 "alpha"=sapply(ss,"[[",4),
-	  class=class(S))
+		 "alpha"=sapply(ss,"[[",4)),
+	  class=class(S)
 	)
 }
 
+#' Spheroid intersection
+#' 
+#' Simulate a spheroid system and intersect
+#' 
+#' The function first simulates a spheroid system according to the parameter \code{theta} and only
+#' returns the resulting section profiles.
+#' 
+#' @param theta simulation parameters
+#' @param cond  conditioning object for simulation and intersection
+#' 
+#' @return list of intersection profiles
+simSpheroidIntersection <- function(theta, cond) {
+	.Call(C_SimulateSpheroidsAndIntersect,
+			c("lam"=cond$lam,theta), cond, cond$nsect)
+}
 
 #' Plot particle system
 #'
@@ -416,7 +441,7 @@ spheroids3d <- function(S, box, draw.axes=FALSE, draw.box=TRUE, draw.bg=TRUE, bg
 	N <- length(S)
 	ll <- lapply(S, function(x)
 				spheroid3d(x$center[1],x$center[2],x$center[3],
-						x$ab[1],x$ab[1],x$ab[2],rotM=x$rotM))
+						x$acb[1],x$acb[2],x$acb[3],rotM=x$rotM))
 
 	rgl::shapelist3d(ll,...)
 
