@@ -326,7 +326,9 @@ namespace STGM {
 
   void CSpheroid::ComputeMatrixA() {
      m_A.nullify();
-	 m_A[0][0] = m_A[1][1] = 1.0 / SQR(m_a);
+	 //m_A[0][0] = m_A[1][1] = 1.0 / SQR(m_a);
+     m_A[0][0] = 1.0 / SQR(m_a);
+     m_A[1][1] = 1.0 / SQR(m_c);
      m_A[2][2] = 1.0 / SQR(m_b);
 
      CMatrix3d R = RotationMatrixFrom001(m_u);
