@@ -20,6 +20,8 @@ extern "C" {
   SEXP FinalizeCylinderSystem(SEXP ext);
   SEXP CylinderSystem(SEXP R_param, SEXP R_cond);
   SEXP CDigitizeCylinderIntersections(SEXP ext, SEXP R_n, SEXP R_z, SEXP R_delta);
+  SEXP IntersectCylinderSystem(SEXP ext, SEXP R_n, SEXP R_z, SEXP R_intern, SEXP R_pl);
+  SEXP SimulateCylindersAndIntersect(SEXP R_param, SEXP R_cond, SEXP R_n);
 
 #ifdef __cplusplus
 }
@@ -31,10 +33,12 @@ namespace STGM {
 class CCylinderSystem {
  public:
 
-  CCylinderSystem(CBox3 &box, double lam, CVector3d &mu )
-      : m_box(box), m_lam(lam), m_maxR(0), m_mu(mu), num(0)
+  CCylinder::cylinder_type m_type;
+
+  CCylinderSystem(CBox3 &box, double lam, CVector3d &mu, CCylinder::cylinder_type type, int perfect = 1 )
+      : m_type(type), m_box(box), m_lam(lam), m_maxR(0), m_mu(mu), num(0), m_perfect(perfect)
   {
-    box.ConstructBoundingPlanes();
+    //box.ConstructBoundingPlanes();
   }
 
   void simSysJoint(R_Calldata d);
@@ -54,6 +58,7 @@ class CCylinderSystem {
   const STGM::CBox3 &box() const { return m_box; }
 
   void IntersectWithPlane(STGM::Intersectors<STGM::CCylinder>::Type &objects, STGM::CPlane &plane, int intern);
+  Rboolean isPerfect() { return (m_perfect > 0 ? TRUE : FALSE); }
 
   private:
     CBox3 m_box;
@@ -62,6 +67,7 @@ class CCylinderSystem {
     size_t num;
 
     Cylinders m_cylinders;
+    int m_perfect;
 
 };
 
